#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Indigo Texecom Premier Plugin
# Copyright (c) 2016-2018 Alan Carter. All rights reserved.
#

################################################################################
# Imports
################################################################################
import socket
import datetime
import binascii
from time import strftime
import indigoPluginUpdateChecker

################################################################################
# Globals
################################################################################

########################################
def updateVar(name, value, folder=0):
	if name not in indigo.variables:
		indigo.variable.create(name, value=value, folder=folder)
	else:
		indigo.variable.updateValue(name, value)

################################################################################
class Plugin(indigo.PluginBase):
	########################################
	# Class properties
	########################################
	
	########################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs): 
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		self.texIP = pluginPrefs.get("ip_address", "")
		self.texPort = pluginPrefs.get("ip_port", "10001")
		self.maxPartitionsString = pluginPrefs.get("max_partitions", "1")
		self.firstZoneString = pluginPrefs.get("first_zone", "1")
		self.lastZoneString = pluginPrefs.get("last_zone", "8")
		self.maxPcOutputsString = pluginPrefs.get("max_pc_outputs", "0")
		self.udlPassword = pluginPrefs.get("udl_pass", "1234")
		self.logComms = pluginPrefs.get("logComms", False)
		self.logEnable = pluginPrefs.get("logLog", False)
		self.updater = indigoPluginUpdateChecker.updateChecker(self, "https://www.dropbox.com/s/qxoo4cqtv4kgrcm/versionInfo.html?dl=1", 7)
	
	########################################
	def __del__(self):
		indigo.PluginBase.__del__(self)
		
	########################################
	def startup(self):
		indigo.server.log("Starting Texecom plugin")
		if self.texIP == "":
			indigo.server.log("Please enter Texecom panel IP in Config and restart plugin", isError=True)
			return
		self.startup = True
		self.comState = "Off"
		self.comFail = False
		self.retries = 0
		self.timeUpdateRequired = True
		self.lastVoltageCheck = 0
		self.updater.checkVersionPoll()
		indigo.server.log("Zones "+self.firstZoneString+" to "+self.lastZoneString+" will be polled")
		if self.logComms:
			indigo.server.log("Panel comms logging is on")
		else:
			indigo.server.log("Panel comms logging is off")			
		if self.logEnable:
			indigo.server.log("Panel events logging is on")
		else:
			indigo.server.log("Panel events logging is off")
		self.createDevices()
		
	def deviceStartComm(self, dev):
		dev.stateListOrDisplayStateIdChanged()
		return

	def shutdown(self):
		indigo.server.log("Stopping Texecom plugin")
		self.comState = "Off"
		self.panelDevice.updateStateOnServer(key="Connection", value="Off")
		self.sock.close()
		pass
		
	########################################
	def runConcurrentThread(self):
		indigo.server.log("Starting Texecom monitoring thread")
		if self.texIP == "":
			indigo.server.log("Please enter Texecom panel IP in Config and restart plugin", isError=True)
			return
		try:
			while True:
				self.sleep(1)
				self.pollPanel()
				self.updater.checkVersionPoll()
				
		except self.StopThread:
			self.comState = "Off"
			try:
				self.panelDevice.updateStateOnServer(key="Connection", value="Off")
				self.sock.close()
			except:
				pass
		
	
	########################################
	# Called when user prefs are changed
	
	def closedPrefsConfigUi(self, valuesDict, userCancelled):
		# If the user saved preferences, update logging parameters
		if userCancelled == False:
			changed = False
			fZone = int(valuesDict.get("first_zone", 1))
			lZone = int(valuesDict.get("last_zone", 1))
			if self.firstZone != fZone:
				self.firstZone = fZone
				changed = True
				indigo.server.log("First Zone changed to "+str(fZone))
			if self.lastZone != lZone:
				self.lastZone = lZone
				changed = True
				indigo.server.log("Last Zone changed to "+str(lZone))
			if changed:
				indigo.server.log("Zones "+str(fZone)+" to "+str(lZone)+" will be polled")
				changed = False
			logComms = self.logComms
			self.logComms = valuesDict.get("logComms", False)
			logEnable = self.logEnable
			self.logEnable = valuesDict.get("logLog", False)
			if self.logComms != logComms:
				if self.logComms:
					indigo.server.log("Panel comms logging is on")
				else:
					indigo.server.log("Panel comms logging is off")
			if logEnable != self.logEnable:
				if self.logEnable:
					indigo.server.log("Panel events logging is on")
				else:
					indigo.server.log("Panel events logging is off")
				

	########################################
	# Texecom specific functions
	
	def createDevices(self):
		""" Create devices for panel, partitions, zones and outputs if not present """
		self.maxPartitions = int(self.maxPartitionsString)
		if 0 < self.maxPartitions > 4:
			# Premier 48 panel supports 4 partitions max
			indigo.server.log("Must specify between 1 and 4 partitions", isError=True)
			return
		self.firstZone = int(self.firstZoneString)
		if 0 < self.firstZone < 1:
			indigo.server.log("First Zone number must be 1 or higher", isError=True)
			return
		self.lastZone = int(self.lastZoneString)
		if 0 < self.lastZone > 48:
			indigo.server.log("Last Zone number must be 48 or lower", isError=True)
			return
		self.maxPcOutputs = int(self.maxPcOutputsString)
		if 0 < self.maxPcOutputs > 4:
			indigo.server.log("Must specify between 0 and 4 PC outputs", isError=True)
			return
		
		# Create Panel Device
		self.panelDevice = None
		pointId = self.texIP
		for dev in indigo.devices.iter("self"):
			if dev.address == pointId:
				self.panelDevice = indigo.devices[dev.name]	
		if self.panelDevice == None:
			pointName = "Texecom Panel"
			indigo.server.log("Creating %s device" % pointName)
			self.panelDevice = indigo.device.create(protocol=indigo.kProtocol.Plugin,
	  		address=pointId,
	   		name=pointName,
	   		pluginId="com.racarter.indigoplugin.texecom",
	    	deviceTypeId="alarmPanel",
	    	props={})
	    			
		# Create Partition Device(s)
		for idx in range(0, self.maxPartitions):
			device = None
			pointId = "TP"+str(idx+1)
			for dev in indigo.devices.iter("self"):
				if dev.address == pointId:
					device = indigo.devices[dev.name]	
			if device == None:
				pointName = "Texecom Partition "+str(idx+1)
				indigo.server.log("Creating %s device" % pointName)
				device = indigo.device.create(protocol=indigo.kProtocol.Plugin,
				address=pointId,
				name=pointName,  
		   		pluginId="com.racarter.indigoplugin.texecom",
		    	deviceTypeId="alarmPartition",
		    	props={})

		 # Create Zone Device(s)   	
		for idx in range(self.firstZone - 1, self.lastZone):
			device = None
			pointId = "TZ"+str(idx+1)
			for dev in indigo.devices.iter("self"):
				if dev.address == pointId:
					device = indigo.devices[dev.name]	
			if device == None:
				pointName = "Texecom Zone "+str(idx+1)
				indigo.server.log("Creating %s device" % pointName)
				device = indigo.device.create(protocol=indigo.kProtocol.Plugin,
				address=pointId,
				name=pointName,  
		   		pluginId="com.racarter.indigoplugin.texecom",
		    	deviceTypeId="alarmZone",
		    	props={})
		    	
		 # Create PC Output Device(s)
		if self.maxPcOutputs > 0:	
			for idx in range(0, self.maxPcOutputs):
				device = None
				pointId = "PC"+str(idx+1)
				for dev in indigo.devices.iter("self"):
					if dev.address == pointId:
						device = indigo.devices[dev.name]	
				if device == None:
					pointName = "PC Output "+str(idx+1)
					indigo.server.log("Creating %s device" % pointName)
					device = indigo.device.create(protocol=indigo.kProtocol.Plugin,
					address=pointId,
					name=pointName,  
			   		pluginId="com.racarter.indigoplugin.texecom",
			    	deviceTypeId="pcOutput",
			    	props={})
		    	
		# Create Power Supply Status Device 	
		device = None
		pointId = "TPower"
		for dev in indigo.devices.iter("self"):
			if dev.address == pointId:
				device = indigo.devices[dev.name]	
		if device == None:
			pointName = "Texecom Mains Power"
			indigo.server.log("Creating %s device" % pointName)
			device = indigo.device.create(protocol=indigo.kProtocol.Plugin,
			address=pointId,
			name=pointName,  
	   		pluginId="com.racarter.indigoplugin.texecom",
	    	deviceTypeId="powerStatus",
	    	props={})
		    	
		self.panelDevice.updateStateOnServer(key="Connection", value="Off")
		self.comState = "Off"
		self.sendingCommand = False

		
		
	def requestAccess(self):
		""" Request access to panel using UDL Password """
		TCP_IP = self.texIP
		TCP_PORT = int(self.texPort)
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.settimeout(15)
		accessRequest = '\\W'+self.udlPassword+'/'
		reply = ""
		if self.logComms:
			indigo.server.log("> "+str(binascii.hexlify(accessRequest)).upper())
		try:
			self.sock.connect((TCP_IP, TCP_PORT))
			self.sock.send(accessRequest)
			self.sleep(1)
			reply = self.sock.recv(1024)
			if self.logComms:
				indigo.server.log("< "+str(binascii.hexlify(reply)).upper())
		except socket.error, v:
			pass
		try:
			self.sock.send(accessRequest)
			self.sleep(1)
			reply = self.sock.recv(1024)
			if self.logComms:
				indigo.server.log("< "+str(binascii.hexlify(reply)).upper())
			self.retries = 0
		except socket.error, v:
			self.retries += 1
			if self.retries == 10:
				if not self.comFail:
					errorcode=str(v[0])
					indigo.server.log("Socket error " + errorcode + " after 10 retries", isError=True)
					self.comFail = True
					self.panelDevice.updateStateOnServer(key="Communications", value="Failed")
			self.sleep(5)
			pass

		if reply.find("OK") != -1:
			if self.startup:	
				indigo.server.log("Panel access granted")
				self.startup = False
			self.panelDevice.updateStateOnServer(key="Connection", value="On")
			self.panelDevice.updateStateOnServer(key="Communications", value="Healthy")
			self.comState = "On"
			self.comFail = False
		elif reply.find("ERROR") != -1:
			indigo.server.log("Panel access error", isError=True)


	def pollPanel(self):
		""" Main loop """
		if self.comState == "Off":
			self.requestAccess()
			self.getPanelInfo()
		else:
			self.polling = True
			self.pollPartitions()
			self.pollZones()
			if self.maxPcOutputs > 0:
				self.pollOutputs()
			self.pollDisplay()
			self.pollSystemStatus()
			self.checkLog()
			self.readVoltages()
			self.checkTime()
			self.polling = False
			
	
	def pollPartitions(self):
		""" Get status of all partitions """
		replyP = self.getResponse('\\P\x00\x18/')
		if len(replyP) > 47:
			alarmStatus = replyP[0:1]
			readyStatus = replyP[32:33]
			fullStatus = replyP[44:45]
			partStatus = replyP[46:47]
			device = None
			pointId = "TP1"
			for dev in indigo.devices.iter("self"):
				if dev.address == pointId:
					device = indigo.devices[dev.name]
			if device != None:
				if map(ord, alarmStatus)[0] & 0x01:
					device.updateStateOnServer(key="alarm_state", value="Tripped")
				else:
					device.updateStateOnServer(key="alarm_state", value="Healthy")
				if map(ord, readyStatus)[0] & 0x01:
					device.updateStateOnServer(key="arm_ready", value="Ready")
				else:
					device.updateStateOnServer(key="arm_ready", value="Not_Ready")
				if map(ord, fullStatus)[0] & 0x01:
					device.updateStateOnServer(key="arm_state", value="Full")
				elif map(ord, partStatus)[0] & 0x01:
					device.updateStateOnServer(key="arm_state", value="Part")
				else:
					device.updateStateOnServer(key="arm_state", value="Disarmed")	
			device = None
			if self.maxPartitions > 1:
				pointId = "TP2"
				for dev in indigo.devices.iter("self"):
					if dev.address == pointId:
						device = indigo.devices[dev.name]
				if device != None:
					if map(ord, alarmStatus)[0] & 0x02:
						device.updateStateOnServer(key="alarm_state", value="Tripped")
					else:
						device.updateStateOnServer(key="alarm_state", value="Healthy")	
					if map(ord, readyStatus)[0] & 0x02:
						device.updateStateOnServer(key="arm_ready", value="Ready")
					else:
						device.updateStateOnServer(key="arm_ready", value="Not_Ready")
					if map(ord, fullStatus)[0] & 0x02:
						device.updateStateOnServer(key="arm_state", value="Full")
					elif map(ord, partStatus)[0] & 0x02:
						device.updateStateOnServer(key="arm_state", value="Part")
					else:
						device.updateStateOnServer(key="arm_state", value="Disarmed")
			device = None
			if self.maxPartitions > 2:
				pointId = "TP3"
				for dev in indigo.devices.iter("self"):
					if dev.address == pointId:
						device = indigo.devices[dev.name]
				if device != None:
					if map(ord, alarmStatus)[0] & 0x04:
						device.updateStateOnServer(key="alarm_state", value="Tripped")
					else:
						device.updateStateOnServer(key="alarm_state", value="Healthy")	
					if map(ord, readyStatus)[0] & 0x04:
						device.updateStateOnServer(key="arm_ready", value="Ready")
					else:
						device.updateStateOnServer(key="arm_ready", value="Not_Ready")
					if map(ord, fullStatus)[0] & 0x04:
						device.updateStateOnServer(key="arm_state", value="Full")
					elif map(ord, partStatus)[0] & 0x04:
						device.updateStateOnServer(key="arm_state", value="Part")
					else:
						device.updateStateOnServer(key="arm_state", value="Disarmed")
			device = None
			if self.maxPartitions > 3:
				pointId = "TP4"
				for dev in indigo.devices.iter("self"):
					if dev.address == pointId:
						device = indigo.devices[dev.name]
				if device != None:
					if map(ord, alarmStatus)[0] & 0x08:
						device.updateStateOnServer(key="alarm_state", value="Tripped")
					else:
						device.updateStateOnServer(key="alarm_state", value="Healthy")	
					if map(ord, readyStatus)[0] & 0x08:
						device.updateStateOnServer(key="arm_ready", value="Ready")
					else:
						device.updateStateOnServer(key="arm_ready", value="Not_Ready")
					if map(ord, fullStatus)[0] & 0x08:
						device.updateStateOnServer(key="arm_state", value="Full")
					elif map(ord, partStatus)[0] & 0x08:
						device.updateStateOnServer(key="arm_state", value="Part")
					else:
						device.updateStateOnServer(key="arm_state", value="Disarmed")

		
	def pollZones(self):
		""" Get status of all zones """
		zoneStatusRequest = '\\Z\x00\x08/'			# Up to 8 zones
		if self.lastZone > 8:
			zoneStatusRequest = '\\Z\x00\x20/'		# Up to 32 zones
		replyZ = self.getResponse(zoneStatusRequest)
		if len(replyZ) > 16:
			for idx in range(self.firstZone - 1, self.lastZone):
				start = idx*2
				end = start + 1
				zoneStatus = replyZ[start:end]
				device = None
				pointId = "TZ"+str(idx+1)
				for dev in indigo.devices.iter("self"):
					if dev.address == pointId:
						device = indigo.devices[dev.name]
				if device != None:	
					if map(ord, zoneStatus)[0] & 0x03 == 0x01:
						device.updateStateOnServer(key="statusText", value="Tripped")
						device.updateStateOnServer(key="status", value=True)
						device.updateStateImageOnServer(indigo.kStateImageSel.MotionSensorTripped)
						tripTime = strftime("%d-%m-%y %H:%M:%S")
						device.updateStateOnServer(key="lastTripped", value=tripTime)
					elif map(ord, zoneStatus)[0] & 0x03 == 0x02:
						device.updateStateOnServer(key="statusText", value="Tamper")
						device.updateStateOnServer(key="status", value=True)
						device.updateStateImageOnServer(indigo.kStateImageSel.MotionSensorTripped)
					elif map(ord, zoneStatus)[0] & 0x03 == 0x03:
						device.updateStateOnServer(key="statusText", value="Shorted")
						device.updateStateOnServer(key="status", value=True)
						device.updateStateImageOnServer(indigo.kStateImageSel.MotionSensorTripped)
					else:
						device.updateStateOnServer(key="statusText", value="Healthy")
						device.updateStateOnServer(key="status", value=False)
						device.updateStateImageOnServer(indigo.kStateImageSel.MotionSensor)
							
							
	def pollOutputs(self):
		""" Read panel PC output status """
		replyOut = self.getResponse('\\O?/')
		if len(replyOut) > 2:
			outputStatus = replyOut[0:1]
			for idx in range(0, self.maxPcOutputs):
				if idx == 0:
					mask = 0x01
				elif idx == 1:
					mask = 0x02
				elif idx == 2:
					mask = 0x04
				elif device == 3:
					mask = 0x08
				device = None
				pointId = "PC"+str(idx+1)
				for dev in indigo.devices.iter("self"):
					if dev.address == pointId:
						device = indigo.devices[dev.name]
				if device != None:	
					if map(ord, outputStatus)[0] & mask == 0x01:
						device.updateStateOnServer(key="onOffState", value=True)
					else:
						device.updateStateOnServer(key="onOffState", value=False)
					

	def pollDisplay(self):
		""" Reads both lines of keypad display """
		replyL = self.getResponse('\\L/')
		if len(replyL) > 30:
			displayLine1 = replyL[0:15]
			displayLine1 = displayLine1.strip()
			displayLine2 = replyL[16:31]
			displayLine2 = displayLine2.strip()
			self.panelDevice.updateStateOnServer(key="keypadDisplay1", value=displayLine1)
			self.panelDevice.updateStateOnServer(key="keypadDisplay2", value=displayLine2)


	def getPanelInfo(self):
		""" Gets panel type and firmware version """
		replyI = self.getResponse('\\I/')
		if len(replyI) > 20:	
			self.panelDevice.updateStateOnServer(key="PanelDetails", value=replyI)

			
	def checkLog(self):
		""" Check the log pointer and get new entries """
		replyG0 = self.getResponse('\\G0/')
		if len(replyG0) > 2:
			logLd = int(binascii.hexlify(replyG0[0:1]), 16)
			logHd = int(binascii.hexlify(replyG0[1:2]), 16)
			logPointer = (logHd * 256) + logLd
			oldPointer = self.panelDevice.states["LogPointer"]
			if logPointer != oldPointer:
				startPointer = max(oldPointer, logPointer -5)
				for lp in range(startPointer, logPointer):
					self.getLog(lp)
				self.panelDevice.updateStateOnServer(key="LogPointer", value=logPointer)
				

	def getLog(self, logPtr):
		""" Get log entries """
		logP = str("0x{:04x}".format(logPtr)[2:])
		logR = binascii.unhexlify(logP[2:4] + logP[0:2])
		replyG2 = self.getResponse('\\G2'+logR+'/')
		if len(replyG2) > 20:
			self.panelDevice.updateStateOnServer(key="LastLogEntry", value=replyG2)
			if "User" in replyG2:
				self.panelDevice.updateStateOnServer(key="LastUser", value=replyG2.split("User", 1)[1][4:])
			if self.logEnable:	
				indigo.server.log(replyG2)
	
	
	def pollSystemStatus(self):
		""" Reads system status Block 0 from panel """
		replyS = self.getResponse('\\S\x00\x01/')
		if len(replyS) > 2:
			systemStatus = replyS[0:1]
			#indigo.server.log(str(systemStatus))
			device = None
			pointId = "TPower"
			for dev in indigo.devices.iter("self"):
				if dev.address == pointId:
					device = indigo.devices[dev.name]
			if map(ord, systemStatus)[0] & 0x02:
				device.updateStateOnServer(key="MainsPower", value="Failed")
				device.setErrorStateOnServer("Failed")
				device.updateStateImageOnServer(indigo.kStateImageSel.PowerOff)
			else:
				device.updateStateOnServer(key="MainsPower", value="OK")
				device.setErrorStateOnServer(None)
				device.updateStateImageOnServer(indigo.kStateImageSel.PowerOn)
				
			if map(ord, systemStatus)[0] & 0x20:
				self.panelDevice.updateStateOnServer(key="PanelTamper", value="Active")
				self.panelDevice.setErrorStateOnServer("Failed")
			else:
				self.panelDevice.updateStateOnServer(key="PanelTamper", value="Secure")
				self.panelDevice.setErrorStateOnServer(None)

			if map(ord, systemStatus)[0] & 0x08:
				self.panelDevice.updateStateOnServer(key="BellTamper", value="Active")
				self.panelDevice.setErrorStateOnServer("Failed")
			else:
				self.panelDevice.updateStateOnServer(key="BellTamper", value="Secure")
				self.panelDevice.setErrorStateOnServer(None)

					
	def readVoltages(self):
		""" Reads system voltages and currents from panel """
		dt = datetime.datetime.now()
		tensCount = int(dt.minute / 10)		# Check every 10 minutes
		if (self.lastVoltageCheck != tensCount):
			replyV = self.getResponse('\\V/')
			if len(replyV) > 2:
				refV = int(binascii.hexlify(replyV[0:1]), 16)
				sysV = int(binascii.hexlify(replyV[1:2]), 16)
				batV = int(binascii.hexlify(replyV[2:3]), 16)
				sysC = int(binascii.hexlify(replyV[3:4]), 16)
				batC = int(binascii.hexlify(replyV[4:5]), 16)
				sysVoltage = 13.7 + ((sysV - refV) * 0.070)
				batVoltage = 13.7 + ((batV - refV) * 0.070)				
				sysCurrent = sysC * 9
				batCurrent = batC * 9
				
				keyValueList = [
         			{'key':'SystemVoltage', 'value':sysVoltage},
         			{'key':'BatteryVoltage', 'value':batVoltage},
         			{'key':'SystemCurrent', 'value':sysCurrent},
         			{'key':'BatteryCurrent', 'value':batCurrent}
      			]
      			self.panelDevice.updateStatesOnServer(keyValueList)
      			
			self.lastVoltageCheck = tensCount

	
	def checkTime(self):
		""" Daily sync of panel time with Indigo time """
		dt = datetime.datetime.now()
		if (dt.hour == 3) and self.timeUpdateRequired:
			self.setPanelTime()
			self.timeUpdateRequired = False
		elif (dt.hour == 4) and self.timeUpdateRequired == False:
			self.timeUpdateRequired = True
			
			
	def setPanelTime(self):
		""" Syncronises panel time with Indigo time """
		dt = datetime.datetime.now()
		mystr=""
		mystr+=format(dt.day, '02X')
		mystr+=format(dt.month, '02X')
		mystr+=format((dt.year - 2000), '02X')
		mystr+=format(dt.hour, '02X')
		mystr+=format(dt.minute, '02X')
		sendStr = binascii.unhexlify(mystr)
		replyT = self.getResponse('\\T'+sendStr+'/')
		if replyT.find("OK") != -1:
			indigo.server.log("Panel time synchronised with Indigo")
		else:
			indigo.server.log("Panel time sync failed", isError=True)


	def getResponse(self, cmdString):
		""" Sends a command to the panel and returns the response """
		panelResponse = ""
		if not self.sendingCommand:
			if self.logComms:
				indigo.server.log("> "+str(binascii.hexlify(cmdString)).upper())
			try:
				self.sock.send(cmdString)
				self.sleep(0.5)
				panelResponse = self.sock.recv(1024)
			except socket.error, v:
				self.comState = "Off"
				self.sock.close()
				pass
			if self.logComms:
				indigo.server.log("< "+str(binascii.hexlify(panelResponse)).upper())
		return panelResponse
				
				

	########################################
	# Action Item functions
	######################
	
	def setArming(self, pluginAction):
		""" Action item to arm/disarm a specified partition """
		self.sendingCommand = True
		if self.polling:
			#indigo.server.log("Polling!")
			self.sleep(1)
		parString = ""
		devToProcess = pluginAction.deviceId
		#indigo.server.log(str(devToProcess))
		device = None
		for dev in indigo.devices.iter("self"):
			if dev.id == devToProcess:
				device = int(dev.address[2:])
				dName = dev.name
				#indigo.server.log(str(device))
				if device != None:
					parString+=format(device, '02X')
					partition = binascii.unhexlify(parString)
					if pluginAction.props["armType"] == "disarm":
						stem = "D"
						#indigo.server.log("Disarming "+dName)
					elif pluginAction.props["armType"] == "fullArm":
						stem = "A"
						#indigo.server.log("Full Arming "+dName)
					elif pluginAction.props["armType"] == "partArm":
						stem = "Y"
						#indigo.server.log("Part Arming "+dName)
					else:
						indigo.server.log("Invalid arm state for "+dName, isError=True)
						self.sendingCommand = False
				
					if self.sendingCommand:
						replyA = ""
						try:
							self.sock.send('\\'+stem+partition+'/')	
							self.sleep(0.5)
							replyA = self.sock.recv(1024)
						except socket.error, v:
							self.comState == "Off"
							self.sock.close()
							pass
						if replyA.find("OK") != -1:
							self.panelDevice.updateStateOnServer(key="LastUser", value="Indigo")
						else:
							indigo.server.log("Command failed", isError=True)
				
					self.sendingCommand = False
					

	def actionControlDevice(self, pluginAction):
		""" Action item to set a specified panel PC Output """
		self.sendingCommand = True
		if self.polling:
			#indigo.server.log("Polling!")
			self.sleep(1)
		parString = ""
		mask = 0
		devToProcess = pluginAction.deviceId
		requestedState = pluginAction.deviceAction
		if str(requestedState) == "TurnOn":
			parString = "FF"
			reqState = "On"
		else:
			parString = "00"
			reqState = "Off"
		device = None
		for dev in indigo.devices.iter("self"):
			if dev.id == devToProcess:
				device = int(dev.address[2:])
				dName = dev.name
				if device != None:
					if device == 1:
						mask = 1
					elif device == 2:
						mask == 2
					elif device == 3:
						mask == 4
					elif device == 4:
						mask = 8
					#indigo.server.log(parString, str(mask))
					parString+=format(mask, '02X')
					outputCommand = binascii.unhexlify(parString)
					stem = "OP"			
					if self.sendingCommand:
						replyA = ""
						try:
							self.sock.send('\\'+stem+outputCommand+'/')	
							self.sleep(0.5)
							replyA = self.sock.recv(1024)
						except socket.error, v:
							self.comState == "Off"
							self.sock.close()
							pass
						#indigo.server.log(str(replyA))
						if replyA.find("OK") != -1:
							indigo.server.log("PC Output "+str(device)+" turned "+reqState)
						else:
							indigo.server.log("PC Output "+str(device)+" command failed", isError=True)
				
					self.sendingCommand = False
					
		


	########################################
	# Menu Item functions
	######################
	
	def setTexTime(self):
		""" Menu option to set panel time to Indigo time """
		self.setPanelTime()
		
	
	def checkVersionNow(self):
		""" The checkVersionNow() method will call the Indigo Plugin Update
        Checker following a user request. """
		self.updater.checkVersionNow()
        
      